// safeccheck.h
// checking function macros #included into box'd files

#ifdef _MSVC
  #ifdef _GNUCC
    #error Exactly one of _MSCV and _GNUCC must be defined.
  #else
    #define FILEFUNC __FILE__
    #define LINEFUNC __LINE__
    #define CHECK_STATIC
     /* Must restrict packing on 32-bit boundary because otherwise the
      * length and tag fields are not adjacent to the data */
    #pragma pack(4)
    #define NORETURN __declspec(noreturn)
    #define NAKED    __declspec(naked)
  #endif
#else
  #ifdef _GNUCC
    #define FILEFUNC (__FILE__ ":" __FUNCTION__)
    #define LINEFUNC __LINE__
    #undef  _GNUCC
    #define _GNUCC 1
    #define CHECK_STATIC static
    #define NORETURN __attribute__((noreturn))
    #define NAKED
 #else
    #error Exactly one of _MSVC or _GNUCC must be defined.
 #endif
#endif

#define I32 int
#define U32 unsigned I32

#ifndef MIN
#define MIN(x,y) (((x) < (y)) ? (x) : (y))
#endif

#define FAIL_BELOW_STACK_ADDRESS 0
#define FAIL_STACK_ADDRESS       1
#define FAIL_LBOUND              2
#define FAIL_UBOUND              3
#define FAIL_FUNCTIONPOINTER     4
#define FAIL_DECR_FSEQ           5
#define FAIL_NULL                6
#define FAIL_NULL_STRING         7
#define FAIL_STRINGBOUND         8
#define FAIL_UNALIGNED           9
#define FAIL_MALLOC              10
#define FAIL_REALLOC             11
#define FAIL_FREE                12
#define FAIL_REALLOC_SHRINK      13
#define FAIL_REALLOC_GROW        14
#define FAIL_NONPOINTER          15
#define FAIL_SCALARLOG           16
#define FAIL_NULLSTRING          17
#define FAIL_VA_BADARG           18
#define FAIL_VA_NOSTART          19
#define FAIL_VA_MANY             20
#define FAIL_VA_BADTYPE          21
#define FAIL_UNRECOGNIZEDFORMAT  22
#define NUM_FAIL_CODES           23   /* sm: keep this updated */

// sm: two modes of operation: with and w/o file/line info
// ww: I cannot stand it, there is no way to get this defined
// conditionally! If you really believe there is, email me
// a script for "make testrun/hello" that somehow gets verbose
// fp_fail messages. Until then, I'm always defining this. Especially
// since Scott's performance measurements indicate that it doesn't
// cost anything.
// #ifndef RELEASELIB
  #define FP_FAIL_IS_VERBOSE	// weimer: I want this one, and there
        			// is no other way (?) to specify that
  // sm: my intent is was to specify EXTRAARGS=-DFP_FAIL_IS_VERBOSE, but
  // we can just make it the default for non-RELEASELIB
// #endif
#if defined(FP_FAIL_IS_VERBOSE)
  #define FP_FAIL_IS_VERBOSE
  #define FP_FAIL_EXTRA_PARAMS , char *file, int line
  #define FP_FAIL_EXTRA_ARGS , file, line
  #define FILE_AND_LINE , FILEFUNC, LINEFUNC
  #define FP_FAIL_STR(s) fp_fail_str(s, __FILE__, __LINE__)
  #define FP_FAIL(code) fp_fail(code, __FILE__, __LINE__)
#else
  #undef  FP_FAIL_IS_VERBOSE
  #define FP_FAIL_EXTRA_PARAMS
  #define FP_FAIL_EXTRA_ARGS
  #define FILE_AND_LINE
  #define FP_FAIL_STR(s) fp_fail_str(s)
  #define FP_FAIL(code) fp_fail(code)
#endif

// declaration of our failure functions
// fail with a given error message
NORETURN  void fp_fail_str(char *str  FP_FAIL_EXTRA_PARAMS);
// fail with a given code, which maps to a message
NORETURN  void fp_fail(int msgId  FP_FAIL_EXTRA_PARAMS);

NORETURN void searchLog(unsigned long l  FP_FAIL_EXTRA_PARAMS);

// Special forms of error functions
#ifdef _MSVC
#define searchLog_zero(b, p) \
 __asm { mov  ebx, p };\
 __asm { mov  eax, b };\
 __asm { test eax, eax };\
 __asm { je  searchLog };
#else
#define searchLog_zero(b, p) if(!(b)) searchLog(p);
#endif

// CHECK_GETFRAME: return the frame pointer of the current function;
// we assume (for stacks growing towards 0) that all local variables
// (those on the stack, anyway) are at addresses less-or-equal this value
#if defined(_MSVC)
  #define CHECK_GETFRAME(putwhere) { \
     char *__bpVal; \
     _asm { mov __bpVal, ebp }; \
     (putwhere) = __bpVal; }
#elif defined(_GNUCC) 
  // sm: gcc doesn't inline functions unless the optimizer is turned
  // on, but I really want to be able to test our system without the
  // optimizer; so I'm turning this into a macro, so we get the frame
  // of the *caller*, not the callee
  #define CHECK_GETFRAME(putwhere) (putwhere) = __builtin_frame_address(0);
#endif

#define CHECK_RETURNPTR(p)  __CHECK_RETURNPTR(p  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_RETURNPTR(void *p  FP_FAIL_EXTRA_PARAMS) {
  // Check that this is not  stack address below the current frame
  char* bpVal;
  CHECK_GETFRAME(bpVal);
  //    printf("CHECK_RETURNPTR: p=0x%08lx, BP=0x%08lx\n", p, bpVal);
  if
#ifdef _MSVC
    // _MSVC seems to put the stack at lowest addresses
    (p && (char*)p <= bpVal)
#else
    // Others put the stack up. Check only the first 2M
    (p && ! ((bpVal - (char*)p) >> 21))
#endif
      fp_fail(FAIL_BELOW_STACK_ADDRESS  FP_FAIL_EXTRA_ARGS);
}

#define CHECK_RETURNFATPTR(p,b)  __CHECK_RETURNFATPTR(p, b  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_RETURNFATPTR(void *p, void*b  FP_FAIL_EXTRA_PARAMS) {
  if(b)
    __CHECK_RETURNPTR(p FP_FAIL_EXTRA_ARGS);
}

#define CHECK_STOREPTR(p)  __CHECK_STOREPTR(p  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_STOREPTR(void *p  FP_FAIL_EXTRA_PARAMS) {
  // Check that this is not a stack address
  if(p) {
    char* bpVal;
    int   delta;
    CHECK_GETFRAME(bpVal)
    delta = (bpVal - (char*)p);
    #ifndef DISABLE_HEAPIFY
      // the usual code
      delta >>= 20;  // Look +- 1MB from the frame. We need a better way to do this
      if(delta == 0 || delta == -1) {  // Catch 0 or 1
        fp_fail(FAIL_STACK_ADDRESS  FP_FAIL_EXTRA_ARGS);
      }                
    #else 
      // some code which should have the same performance effect
      // as the above code where the 'if' condition is always false
      delta <<= 4;
      if (delta == 1 || delta == -1) {    // never true (does the compiler know this?)
        fp_fail(FAIL_STACK_ADDRESS  FP_FAIL_EXTRA_ARGS);
      }
    #endif
  }
} 

#define CHECK_STOREFATPTR(p,b)  __CHECK_STOREFATPTR(p, b  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_STOREFATPTR(void *p, void* b  FP_FAIL_EXTRA_PARAMS) {
  if(b) {
    __CHECK_STOREPTR(p  FP_FAIL_EXTRA_ARGS);
  }
}

// Fetch the length in words
#define CHECK_FETCHLENGTH(p,b) __CHECK_FETCHLENGTH(p, b  FILE_AND_LINE)
__inline static unsigned int __CHECK_FETCHLENGTH(void *p, void *b  FP_FAIL_EXTRA_PARAMS) {
  if(!b)
    searchLog((unsigned long)p  FP_FAIL_EXTRA_ARGS);
  return (unsigned int)(* (((U32*)b) - 1));
}

#define CHECK_FUNCTIONPOINTER(p,b,n) __CHECK_FUNCTIONPOINTER(p, b, n  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_FUNCTIONPOINTER(void *p, void *b, int nrActualArgs  FP_FAIL_EXTRA_PARAMS) {
  if(!b)
    searchLog((unsigned long)p  FP_FAIL_EXTRA_ARGS);
  if((* (((U32*)b) - 1)) ||
     (* (((U32*)b) + 0)) != (U32)p ||
     (* (((U32*)b) + 1)) > nrActualArgs) {
    fp_fail(FAIL_FUNCTIONPOINTER  FP_FAIL_EXTRA_ARGS);
  }
}

// Fetch the address of the word following the area
#define CHECK_FETCHEND(p,b) __CHECK_FETCHEND(p, b  FILE_AND_LINE)
__inline static void * __CHECK_FETCHEND(void *p, void *b  FP_FAIL_EXTRA_PARAMS) {
  unsigned int len;
  if(!b)
    searchLog((unsigned long)p  FP_FAIL_EXTRA_ARGS);
  len = (unsigned int)(* (((U32*)b) - 1));
  return (void*)((char*)b + (len << 2));
}

extern unsigned int strlen(const char *);
// Fetch the address of the word following the area
#define CHECK_FETCHSTRINGEND(p) __CHECK_FETCHSTRINGEND(p  FILE_AND_LINE)
__inline static void * __CHECK_FETCHSTRINGEND(char *p  FP_FAIL_EXTRA_PARAMS) {
  unsigned int len;
  /* WEIMER: mod_gzip doesn't seem to like this one, so I'm
   * temporarily changing the semantics to return NULL
   * Wed Oct 24 10:44:47 PDT 2001 */
  if(!p)
    return NULL; 
    // fp_fail(FAIL_NULLSTRING  FP_FAIL_EXTRA_ARGS);
  len = strlen(p); // Do not include the terminating 0 !!!
  len += 1;     // weimer: do include it (for reading) ! XXX! hack for now ...
  return (void*)((char*)p + len);
}

// Check the upper bound:
//   bend - pointer to the first byte the user is *not* allowed to read/write
//   p    - pointer the user is using to read/write
//   plen - length of the user's data being read/written
#define CHECK_UBOUND(bend,p,plen) __CHECK_UBOUND(bend,p,plen  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_UBOUND(void *bend, void *p, unsigned int plen  FP_FAIL_EXTRA_PARAMS)
{                                     
  // sm: Escalating the +1 war again: under the interpretation specified
  // in George's 11/10/2001 email, and consistent with the way box.ml
  // generates code, any access to the 'bend' byte is illegal.  I know this
  // will cause problems, so I want to find where they are and fix them
  // there.
  if ((char*)p + plen > (char*)bend) {
    fp_fail(FAIL_UBOUND  FP_FAIL_EXTRA_ARGS);
  }
  else {
    // no problem
  }
}

// sm: upper bound or null
#define CHECK_UBOUNDNULL(bend,p,plen) __CHECK_UBOUNDNULL(bend,p,plen  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_UBOUNDNULL(void *bend, void *p,
                        unsigned int plen  FP_FAIL_EXTRA_PARAMS)
{
  if (!p) {
    // ok
  }
  else {
    __CHECK_UBOUND(bend, p, plen  FP_FAIL_EXTRA_ARGS);
  }
}

// Check the lower bound
//   base - pointer to the first byte the user is allowed to read/write
//   p    - pointer the user is using to read/write
#define CHECK_LBOUND(bend,p) __CHECK_LBOUND(bend,p  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_LBOUND(void *base, void *p  FP_FAIL_EXTRA_PARAMS)
{
  if (p < base) {
    fp_fail(FAIL_LBOUND  FP_FAIL_EXTRA_ARGS);
  } else {
    // pl
  }
}

#define CHECK_BOUNDS(b,bend,p,plen) __CHECK_BOUNDS(b,bend,p,plen  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_BOUNDS(void *b, void *bend,
                       void *p, unsigned int plen  FP_FAIL_EXTRA_PARAMS) {
  __CHECK_LBOUND(b, p  FP_FAIL_EXTRA_ARGS);
  __CHECK_UBOUND(bend, p, plen  FP_FAIL_EXTRA_ARGS);
}

// sm: check bounds, allowing p to be null
#define CHECK_BOUNDSNULL(b,bend,p,plen) __CHECK_BOUNDSNULL(b,bend,p,plen  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_BOUNDSNULL(void *b, void *bend,
                       void *p, unsigned int plen  FP_FAIL_EXTRA_PARAMS) {
  __CHECK_LBOUND(b, p  FP_FAIL_EXTRA_ARGS);      // lbound already succeeds on null
  __CHECK_UBOUNDNULL(bend, p, plen  FP_FAIL_EXTRA_ARGS);
}

// Cast a seqp to a fseqp
#define CHECK_SEQTOFSEQ(b,p) __CHECK_SEQTOFSEQ(b, p  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_SEQTOFSEQ(void *b, void *p FP_FAIL_EXTRA_PARAMS) {
  __CHECK_LBOUND(b, p FP_FAIL_EXTRA_ARGS);
}

#define CHECK_BOUNDS_LEN(b,bw,p,plen) __CHECK_BOUNDS_LEN(b,bw,p,plen  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_BOUNDS_LEN(void *b, unsigned int bwords,
                      void *p, unsigned int plen  FP_FAIL_EXTRA_PARAMS) {
  __CHECK_BOUNDS(b, (void*)((char*)b + (bwords << 2)), p, plen  FP_FAIL_EXTRA_ARGS);
}

#define CHECK_BOUNDS_INDEX(b,p,plen) __CHECK_BOUNDS_INDEX(b,p,plen  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_BOUNDS_INDEX(void *b, void *p, unsigned int plen  FP_FAIL_EXTRA_PARAMS) {
  __CHECK_BOUNDS(b, CHECK_FETCHEND(p, b), p, plen  FP_FAIL_EXTRA_ARGS);
}

#define CHECK_POSITIVE(x) __CHECK_POSITIVE(x  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_POSITIVE(int x  FP_FAIL_EXTRA_PARAMS) {
  if(x < 0) {
    fp_fail(FAIL_DECR_FSEQ  FP_FAIL_EXTRA_ARGS);
  }
}

#define CHECK_ADVANCE(p, x) __CHECK_ADVANCE(p, x  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_ADVANCE(char *p, int x  FP_FAIL_EXTRA_PARAMS) {
  if(p + x < p) {
    fp_fail(FAIL_DECR_FSEQ  FP_FAIL_EXTRA_ARGS);
  }
}

// sm: let's always turn off the null check to improve performance
// #define OMIT_NULL_CHECK

#define CHECK_NULL(p) __CHECK_NULL(p  FILE_AND_LINE)
#ifdef OMIT_NULL_CHECK
  #define __CHECK_NULL(p)
#else
  __inline CHECK_STATIC
  void __CHECK_NULL(void *p  FP_FAIL_EXTRA_PARAMS) {
    if(!p) searchLog((unsigned int)p  FP_FAIL_EXTRA_ARGS);
  }
#endif


// Check that a string is within bounds
#define CHECK_STRINGMAX(p,b) __CHECK_STRINGMAX(p, b  FILE_AND_LINE)
__inline CHECK_STATIC
unsigned int __CHECK_STRINGMAX(void *p, void* b  FP_FAIL_EXTRA_PARAMS) {
  int nrWords;
  int max;
  //  U32 *addr = (U32*)p;
  if(! b) {
    fp_fail(FAIL_NONPOINTER  FP_FAIL_EXTRA_ARGS);
  }
  if(p < b) {
    fp_fail(FAIL_STRINGBOUND  FP_FAIL_EXTRA_ARGS);
  }
  nrWords = *(((U32*)b) - 1);
  max     =  ((char*)b - (char*)p) + (nrWords << 2);
  if(max <= 0) {
    fp_fail(FAIL_STRINGBOUND  FP_FAIL_EXTRA_ARGS);
  }
  return (unsigned int)max;
}

#define CHECK_STRINGLEN(p,b) __CHECK_STRINGLEN(p, b  FILE_AND_LINE)
__inline CHECK_STATIC
unsigned int __CHECK_STRINGLEN(const void *p, unsigned int max  FP_FAIL_EXTRA_PARAMS) {
  int i = 0;
  const char *x = (const char*)p;
  for(; max; max --, i ++, x ++) {
    if(! *x) {
      return i;
    }
  }
  fp_fail(FAIL_STRINGBOUND  FP_FAIL_EXTRA_ARGS);
  return 0;
}


/************** TAG MANIPULATION ***************/

/* We have one tag bit per aligned data word. The tag bit value is 1 if the
 * data word contains a pointer (i.e. the _p first-half of a fat pointer)
 * and 0 otherwise. A very important invariant is that a tag bit sequence
 * "10" denotes two data words that contain a fat pointer (with the fields
 * _p and _b in that order). To maintain this invariant we have to clear the
 * tag bit for word "w - 1" when we write anything in the word "w". An
 * exception is when we write a whole fat pointer in which case we write the
 * tags "10".  
 *
 * Also note that we're counting bits starting at the right edge and moving
 * left.  So, if I have a block of memory containing one fat pointer, with
 * a valid base and ptr, it looks like:
 *
 *  <-- 32 --> <-- 32 --> <-- 32 --> <--------------- 32 -------------------->
 *  +---------+----------+----------+-----------------------------------------+
 *  | size: 2 |    ptr   |   base   | 0000 0000 0000 0000 0000 0000 0000 0001 |
 *  +---------+----------+----------+-----------------------------------------+
 *                                                                         ^^
 * The last "1" and "0" are the "10" combination referred to above.
 *
 * (However, we work left to right among the *words* of the tag area.  Also,
 * if the machine is little-endian, the byte containing the value 1 will actually
 * be first in the tag area, since they are written using U32 pointers.)
 */
     
// The number of tags bits per data word
#define TAGBITS          1
#define TAGBITSMASK      ((1 << TAGBITS) - 1)    // 1

// How many tags fit into a 32-bit word
#define TAGSPERWORD_LOG  (5 - TAGBITS + 1)       //  5
#define TAGSPERWORD      (1 << TAGSPERWORD_LOG)  //  0x20

// The tags for a fat pointer
#define FATPTR_TAG       1  // a 1 followed by a 0
#define FATPTR_MASK      (TAGBITSMASK | (TAGBITSMASK << TAGBITS))   // 3


typedef struct tagAddr {
  U32 *word;                  /* This is the address of the word in which 
                               * the tag lives */
  int bit;                    /* This is the index of the bit where the tag 
                               * starts. 0 means the LSB. Will always be 
                               * less or equal 32 - TAGBITS (at least one 
                               * tag still fits)  */
} TAGADDR;


__inline CHECK_STATIC
int CHECK_NRTAGWORDS(unsigned int nrDataWords) {
  return (nrDataWords + TAGSPERWORD - 1) >> TAGSPERWORD_LOG;
}


/* Compute the address of the tag for a given word. Both the base and p must 
 * be aligned on word boundary */
#define CHECK_FETCHTAGADDR(b,bw,p) __CHECK_FETCHTAGADDR(b,bw,p FILE_AND_LINE)
__inline CHECK_STATIC
TAGADDR __CHECK_FETCHTAGADDR(void *base, // Base
                           unsigned int bwords, // words in area
                           void *p  // the pointer
                           FP_FAIL_EXTRA_PARAMS) {
  TAGADDR res;
#ifndef NO_TAGS
  int wrdIdx    = ((char*)p - (char*)base) >> 2;
  res.word      = (U32*)base + bwords + (wrdIdx >> TAGSPERWORD_LOG);
  res.bit       = TAGBITS * (wrdIdx & (TAGSPERWORD - 1));
#else
  fp_fail(FAIL_NOTAGS  FP_FAIL_EXTRA_ARGS);
#endif
  return res;
}

/* Compute the number of bits that correspond to the number of words covered 
 * by the range given. nextAddr is 1 past the last address in the range */
__inline CHECK_STATIC
int CHECK_NRTAGBITS(char* startAddr, char* nextAddr) {
  int startWord = ((int)startAddr) >> 2;
  int nextWord  = ((int)nextAddr + 3) >> 2;
  return (nextWord - startWord) * TAGBITS;
}

/* Zero out the tags for a written memory area. Later we'll come and fix the 
 * tags for the words that contain pointers. We must clear also the tag 
 * for the word that preceedes the first written word. To avoid unnecessary 
 * clearing of tag bits make sure that you pass in offset the offset to the 
 * first scalar field written. This means what we write the tags for the 
 * memory area "where + offset - 4" to "where + size - 1". */
#ifndef NO_TAGS
__inline CHECK_STATIC
void CHECK_ZEROTAGS(void *base,
                    unsigned int nrbwords,
                    void *where,
                    unsigned int size /* in bytes */,
                    unsigned int offset /* in bytes */) {
  char *wherec = (char*)where;

  return;
  
  // Adjust for the offset
  wherec += offset;
  size  -= offset;
  if((void*)wherec > base) { // Adjust for the preceeding word
    wherec -= 4;
    size  += 4;
  }
  {
    TAGADDR stag    = CHECK_FETCHTAGADDR(base, nrbwords, wherec);
    int nrBits      = CHECK_NRTAGBITS(wherec, wherec + size);
    
    if(stag.bit > 0) {
      if(stag.bit + nrBits < 32) { // Don't zero to the end
        *stag.word ++ &= (~ (((1 << nrBits) - 1) << stag.bit));
        nrBits = 0;
      } else {
        *stag.word ++ &= (~ ((0 - 1) << stag.bit));
        nrBits -= (32 - stag.bit);
      }
    }
    while(nrBits >= 32) {
      *stag.word ++ = (U32)0;
      nrBits -= 32;
    }
    if(nrBits > 0) { // Some leftover in the last word
      *stag.word &= (~ ((1 << nrBits) - 1));
    }
  }
}
#else
#define CHECK_ZEROTAGS(b,w,l,t,o) ; 
#endif

/* Copy the tags going forward. Call this in conjuction with any memory 
 * copy. The psrc and pdst and size should be _exactly_ as passed to memcpy. 
 * NOTE THAT THIS MIGHT NOT WORK PROPERLY IN CASE OF OVERLAPPING AREAS 
 * (since the bits are read first and then written as a block. To model 
 * overlapping correctly we need to copy one tag at a time.) Note that this 
 * is used in a somewhat unusual way in realloc.  */
void CHECK_COPYTAGSFORW(void *bsrc, /* base of src */
                        unsigned int lensrc, /* len of src (in words) */
                        char* psrc, /* pointer of src */
                        void *bdst, /* base of dest */
                        unsigned int lendst, /* words in dest*/
                        char* pdst, /* pointer of dest */
                        unsigned int size /* Size in bytes 
                                           * of the memory 
                                           * being copied */);

/* Copy the tags going backward. The psrc, pdst and size arguments should be 
 * exactly as passed to memcpy. The copying is done going backwards from the 
 * last word. NOTE THAT THIS MIGHT NOT WORK PROPERLY IN CASE OF OVERLAPPING 
 * AREAS (since the bits are read first and then written as a block. To 
 * model overlapping correctly we need to copy one tag at a time.) Note that 
 * this is used in a somewhat unusual way in realloc.  */
void CHECK_COPYTAGSBACK(void *bsrc, /* base of src */
                        unsigned int lensrc, /* len of src (in words) */
                        char* psrc, /* pointer of src */
                        void *bdst, /* base of dest */
                        unsigned int lendst, /* words in dest*/
                        char* pdst, /* pointer of dest */
                        unsigned int size);


#define CHECK_FATPOINTERREAD(b,n,w) __CHECK_FATPOINTERREAD(b,n,w  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_FATPOINTERREAD(void *base, unsigned int nrWords,
                            void* where  FP_FAIL_EXTRA_PARAMS) {
#ifndef NO_TAGS
  TAGADDR tag  = CHECK_FETCHTAGADDR(base, nrWords, where);
  U32 diffTag, memTag;
  
  if((U32)where & 0x3) {
    fp_fail(FAIL_UNALIGNED  FP_FAIL_EXTRA_ARGS);
  }
  
  memTag  = (* tag.word ++) >> tag.bit;
  
  if(tag.bit == (32 - TAGBITS)) { // Only the tag for the first half
    memTag |= ((* tag.word) << TAGBITS);
  }
  diffTag = (memTag & FATPTR_MASK) ^ FATPTR_TAG;
  if(diffTag == 0)
    return;
  // We give it one more chance. Maybe the word that would be read for base
  // is 0, in which case everything is still safe. This will handle the case
  // when an area has been initialized with 0 and the tags are all 0

  /* Note that this is not quite right. We should not be writing in the 
   * memory. Instead we should be just pretending to load a 0 instead of the 
   * base */
  *(((U32*)where) + 1) = (U32)0;
  return;
#endif
}



/* We are writing (changing the value of) a fat pointer
 * which, itself, lives inside a tagged block of memory */
#define CHECK_FATPOINTERWRITE(b,n,w,x,y) __CHECK_FATPOINTERWRITE(b,n,w,x,y  FILE_AND_LINE)
__inline CHECK_STATIC
void __CHECK_FATPOINTERWRITE(
  // the base of the block in which the fat pointer lives
  // (the byte just after the length)
  void *base,

  // the number of words in the block containing
  // the fat pointer
  unsigned int nrWords,

  // the address of the fat pointer itself; this will
  // point to the '_p' field, since it is first
  void *where,

  // 'whatbase' is the value we want the fat pointer's
  // '_b' (base) field to become
  void *whatbase,

  // and this is what we want its '_p' field to become
  void *whatp
       
  FP_FAIL_EXTRA_PARAMS)
{
#ifndef NO_TAGS
  // compute bit address of tag bit
  TAGADDR tag  = CHECK_FETCHTAGADDR(base, nrWords, where);

  // sm: maybe this alignment check should go first?
  if((U32)where & 0x3) {
    fp_fail(FAIL_UNALIGNED  FP_FAIL_EXTRA_ARGS);
  }
                  
  // mask out the existing tag bits, and OR in the "10"
  // for a valid fat pointer
  *tag.word = (* tag.word & (~ (FATPTR_MASK << tag.bit))) |
                  (FATPTR_TAG << tag.bit);

  if(tag.bit < 32 - TAGBITS) // Both fit in the same word
    goto afterTagCheck;    // sm: used to say 'return;'
    
  // the "10" straddles a word boundary, so the "0" didn't
  // actually get written above (left-shift just discards
  // extra bits); so we now write that last "0" into the
  // next tag word
  tag.word ++;
  *tag.word = (*tag.word & (~ (FATPTR_MASK >> TAGBITS))) |
                 (FATPTR_TAG >> TAGBITS);
                 
afterTagCheck:
#endif
  if(whatbase) {
    CHECK_STOREPTR(whatp);
  }
  return;
}


/* weimer: our hooks into the splay tree data structure */
#include "splay.h"

// here, 'end' is the last+1 byte
__inline CHECK_STATIC
void CHECK_REGISTERAREA(int kind, void *base, void *end) {
    if (kind == registerTagged || kind == registerSized) {
        end = CHECK_FETCHEND(base,base);
    }
    splayInsertInterval(kind, base, (char*)end - (char*)base);
    //splayPrintTree(&__global_tree[kind]);
    return;
}


// sm: changed this to a macro too, so that unregistering works in gcc
// even when optimization is turned off
#define CHECK_UNREGISTERFRAME() {                              \
  char* thisf;                                                 \
  CHECK_GETFRAME(thisf);                                       \
  splayRemoveKindBelow(MR_STACK, registerTagged, thisf);       \
  splayRemoveKindBelow(MR_STACK, registerSized, thisf);        \
  splayRemoveKindBelow(MR_STACK, registerSeq, thisf);          \
}


__inline CHECK_STATIC
void * CHECK_FINDHOME(int kind, void *p) {
  return splayFindHome(kind, p);
}

__inline CHECK_STATIC
void * CHECK_FINDHOMEEND(int kind, void *p, void **e) {
  return splayFindHomeEnd(kind, p, e);
}





