@rem = '--*-Perl-*--';
@rem = '
@echo off
perl -S scaninfer.bat %*
goto endofperl
@rem ';

use strict;

# use Data::Dumper;
use Cwd;
require Getopt::Long;


# Turn all buffering off. To make emacs happy
select((select(STDOUT), $| = 1)[0]);
select((select(STDERR), $| = 1)[0]);
select((select(STDIN),  $| = 1)[0]);


# &Getopt::Long::Configure('pass_through');
# $Getopt::Long::bundling = 1;

# print "ARGV= ", join(' ', @ARGV), "\n";

my (%option);
&Getopt::Long::GetOptions
    (\%option,
     "help",            # Display help information
     "fullname"
     );

if($option{help}) {
    &printHelp();
    exit 0;
}

sub printHelp {
    print << "EOF";
Usage: scaninfer [options]
Options:
  -help                       Display this information
  -file <name>                The file to scan
  -fullname                   Print info in the format expected by emacs
Report bugs to /dev/null
EOF
}

if($#ARGV < 0) {
    die "No file given\n";
}

my $file = $ARGV[0];
# print "Indexing file $file\n";
my $fullpath = $file; # Do this by default
my $isabsolute =
    $^O eq "MSWin32" ? ($file =~ m%^(.:[/\\])|[/\\]%) : ($file =~ m|^/|);
if(! $isabsolute) {
    my $cwd = Cwd::cwd;
    my $pathsep = $^O eq "MSWin32" ? '\\' : '/';
    $fullpath = $cwd . $pathsep . $file;
}
#            
#                
#        
#    
#    open(FULLPATH, "cygpath -wa $file|") ||
#        die "Cannot run cygpath $file";
#    $fullpath = <FULLPATH>;
#    close(FULLPATH) || die "Cannot run cygpath $file\n";
#    chop $fullpath;
#}

open(FIN, "<$file") || die "Cannot read file $file\n";

 # Keep track of information for each node
my %nodes = ();

my $lineno = 0;
my $insolution = 0;
my $thislinepos = 0;
print "Indexing the file $file. Please wait...\n";

while(<FIN>) {
    $lineno ++;
    if(! $insolution) {
        # In the code part
        while($_ =~ m|NODE\((\d+)\)|g) {
            my $nd = $1;
            $nodes{$nd} = { DEFLINE => $lineno,
                            DEFPOS => $thislinepos };
            # print "Found node $nd in $lineno\n";
        }
        if($_ =~ m|Now the solved graph|) {
            $insolution = 1;
        }
    } else {
        # In the solution part
        if($_ =~ m|^(\d+) :(.+)$|) {
            my $nd = $1;
            # print "Found node $nd (sol) in $lineno\n";
            $nodes{$nd}->{SOLLINE} = $lineno;
            $nodes{$nd}->{SOLPOS} = $thislinepos;
        }
    }
    $thislinepos = tell(FIN);
}


&emacsLine($fullpath, 1);
 TopLevel:
    while(1) {
        print "Enter node (q to end): ";
        my $ndid = <STDIN>; chop $ndid;
        if($ndid eq "") { next; }
        if($ndid eq "q") { last TopLevel; }
        my $downid;
        my @stack = ();

      ShowNode:
        my ($thenode, $ndname, $ndkind, $ndwhy, $ndtyp) = &getNodeSol($ndid);
            
        print "\n*** Node $ndid. $ndname. T=$ndtyp.\n";
        my $defnode = $thenode; # The node that is defined
        my $defname = $ndname;
        # Search for a definition
        while(! defined($defnode->{DEFLINE})) {
            # Maybe it is an Offset
            if($defname =~ m|^Offset\((\d+),|) {
                my $parentid = $1;
                my ($pnode, $pname, $pkind, $pwhy, $ptyp) = 
                    &getNodeSol($parentid);
                print " Child of node $parentid. $pname. K=$pkind. T=$ptyp\n";
                $defnode = $pnode;
                $defname = $pname;
            } else {
                print "Don't know where node $ndid is defined\n";
            }
        }
        &emacsLine($fullpath, $defnode->{DEFLINE});
        print "  Kind $ndkind. Because: $ndwhy\n";
        $downid = undef;
        if($ndwhy =~ m|spread_from_edge\((\d+)\)|) {
            $downid = $1;
        }
        if($ndwhy =~ m|spread_points_to\((\d+)\)|) {
            $downid = $1;
        }
        if($ndwhy =~ m|spread_to_array_from\((\d+)\)|) {
            $downid = $1;
        }
        if($ndwhy =~ m|cast\(.+ \((\d+)\) <= .+ \((\d+)\)\)$|) {
            if($ndid == $1) { 
                $downid = $2; 
            } else {
                $downid = $1;
            } 
        }
        while(1) {
            my $downprompt = defined($downid) ? "(d)own, " : "";
            my $upprompt   = $#stack >= 0 ? "(u)p, " : "";
            print "$downprompt$upprompt(q)uit, nodeid: ";
            my $cmd = <STDIN>; chop $cmd;
            
            if(($cmd eq "" || $cmd eq "d") && defined($downid)) {
                push @stack, $ndid;
                $ndid = $downid;
                goto ShowNode;
            }
            if($cmd eq "u" && $#stack >= 0) {
                $ndid = pop @stack;
                goto ShowNode;
            }
            if($cmd eq "q") {
                last TopLevel;
            }
            if($cmd =~ m|\d+|) {
                $ndid = $cmd;
                @stack = ();
                goto ShowNode;
            }
            if($cmd ne "") { print "Invalid command $cmd\n"; }
        }
    }


close(FIN);

sub seekToPos {
    my($linepos) = @_;
    seek(FIN, $linepos, 0) || die "Cannot seek to $linepos";
}


sub emacsLine {
    my ($fname, $line) = @_;
    if($option{fullname}) {
        print "\032\032$fname:$line:\n";
    }
}

sub getNodeSol {
    my ($ndid) = @_;
    if(! defined($nodes{$ndid})) {
        print "Don't know about node $ndid\n"; next;
    }
    my $thenode = $nodes{$ndid};
    # Go to the solution
    &seekToPos($thenode->{SOLPOS});
    $_ = <FIN>; 
    my ($ndname) = ($_ =~ m|\d+ : (.+\.\d+) \(|);
    $_ = <FIN>;
  AnotherSol:
    if($_ =~ m|^\s*K=(\S+)/(.+\S)\s+T=(.+)$|) {
        return ($thenode, $ndname, $1, $2, $3);
    } else {
            # Maybe it spans multiple lines
        if($_ =~ m|^\d+|) { # This must be the next node
            print "Cannot understand solution of node $ndid ($_)\n";
            return ($thenode, $ndname);
        }
        $_ = <FIN>;
        goto AnotherSol;
    }
}

__END__
:endofperl

