(*
 * Copyright  1990-2002 The Regents of the University of California. All rights reserved. 
 *
 * Permission is hereby granted, without written agreement and without 
 * license or royalty fees, to use, copy, modify, and distribute this 
 * software and its documentation for any purpose, provided that the 
 * above copyright notice and the following two paragraphs appear in 
 * all copies of this software. 
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY 
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN 
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. 
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS 
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION 
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *)

(* $Id: fociMisc.mli,v 1.1 2003/06/16 18:52:26 jhala Exp $
 *
 * This file is part of the BLAST Project.
 *)


(**
 * This module provides some miscellaneous useful helper functions.
 *)

(** swap a pair *)
val swap : ('a * 'b) -> ('b * 'a)

(* playing with triples *)
(** get the first element of a triple *)
val fst3 : ('a * 'b * 'c) -> 'a

(** get the second element of a triple *)
val snd3 : ('a * 'b * 'c) -> 'b

(** get the third element of a triple *)
val thd3 : ('a * 'b * 'c) -> 'c


(** adds an element to a list representing a set *)
val add : 'a list -> 'a -> 'a list

(** union two lists representing sets *)
val union : 'a list -> 'a list -> 'a list

(** intersect two lists representing sets *)
val intersection : 'a list -> 'a list -> 'a list

(** removes the duplicate elements of a list *)
val compact : 'a list -> 'a list

(** set difference using lists to represent sets *)
val difference : 'a list -> 'a list -> 'a list

(** set containment using lists *)
val subsetOf : 'a list -> 'a list -> bool

(** given a list of strings, produce a single string that is the
   comma-separated list of the original strings *)
val strList : string list -> string

(** returns the to_string function corresponding to the given
   pretty-printer *)
val to_string_from_printer :
  (Format.formatter -> 'a -> unit) -> 'a -> string

(** given a printer for a type t, returns a printer for lists of elements of
   type t *)
val list_printer_from_printer :
  (Format.formatter -> 'a -> unit) -> Format.formatter -> 'a list -> unit

(** returns true if the char is a digit (0-9) *)
val is_digit : char -> bool

(** a fault tolerant map for lists.
    map_exn takes an exception {i ex} to be caught, a function {i f: 'a -> 'b)} 
    and an {i 'a list}, and produces a {i 'b list} by
    applying function {i f} to each element of the list.
    If evaluating the function raises exception {i ex},
    the corresponding element is omitted in the result.
*)
val map_exn : exn -> ('a -> 'b) -> 'a list -> 'b list

(** given a predicate {i P} and a list {i l}, return two
   sublists of {i l}: those that satisfy {i P}, and those
   that do not.
*)
val filter_cut : ('a -> bool) -> 'a list -> ('a list * 'a list)

(** returns the truncated list containing the first n elements Raise Failure "nth" if list is too short *)
val truncateList : 'a list -> int -> 'a list

(** is_prefix pref str checks if pref is a prefix of string str *)
val is_prefix : string -> string -> bool

(** given a prefix and a string, return the suffix of
    the string after the prefix, i.e., if s = p.s', then
    {i chop_after_prefix p s} is {i s'} *)
val chop_after_prefix : string -> string -> string 

(** Hashtbl union: {[hashtbl_addtable t1 t2]} adds all data of {i t2} to {i t1} *)
val hashtbl_addtable : ('a, 'b) Hashtbl.t -> ('a, 'b) Hashtbl.t -> unit 

(* val search_list : ('a -> 'b option) -> 'a list -> b' option *)

(** given a predicate and a list, break the list at the first point an
    element satisfies the predicate. The element satisfying the predicate
    is in the second part.
*)
val chop_list : ('a -> bool) -> ('a list) -> ('a list * 'a list)

(** Given a list of 'a option, get all the elements of the form Some v *)
val list_gather_option : ('a option list) -> ('a list)

(** find the sum of a list of integers *)
val list_add : int list -> int
