(*
 * Copyright  1990-2002 The Regents of the University of California. All rights reserved. 
 *
 * Permission is hereby granted, without written agreement and without 
 * license or royalty fees, to use, copy, modify, and distribute this 
 * software and its documentation for any purpose, provided that the 
 * above copyright notice and the following two paragraphs appear in 
 * all copies of this software. 
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY 
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN 
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. 
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS 
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION 
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *)

(* $Id: abstraction.mli,v 1.7 2003/07/17 00:30:35 adamc Exp $
 *
 * This file is part of the BLAST Project.
 *)


(**
 * This module implements the abstraction interface
 * connecting the model checker to the concrete transition system.
 * It defines the abstract pre and post, and various routines for
 * focus, shrink, etc used by the model checker.
 *) 


open BlastArch


module Make_C_Abstraction :
  functor(C_SD : SYSTEM_DESCRIPTION with module Command = BlastCSystemDescr.C_Command) ->
  sig
    include ABSTRACTION
    (* val build_custom_abstraction : (((string * ((Ast.Expression.expression * bool) list)) list) 
				      * ((Ast.Expression.expression * (Ast.Expression.expression * bool) list) list option) 
				      * ((Ast.Expression.expression*string) list))-> unit
*)	
    val initKill : C_SD.location option -> Ast.Predicate.predicate list -> unit
    (* create a region: takes a location, a flag that sets the stack (flag = true -> EveryStack, flag = false -> [])
	the set of seed predicates, the initial state and returns a region
	*)
      (** As the name suggests, this sets up the abstraction by loading in predicates and such *)
    val initialize_abstraction : unit -> unit
    val dump_abs : unit -> unit
    val create_region : C_SD.location list -> bool -> Ast.Predicate.predicate -> Ast.Predicate.predicate -> Region.t
    val create_error_region : C_SD.location -> int -> Ast.Predicate.predicate -> Region.t
    val print_stats : Format.formatter -> unit -> unit
    val dumpXml : C_SD.edge list -> unit
    val check_races : bool ref
    val make_const_table : unit -> unit
    val assumed_invariants : Ast.Predicate.predicate list ref
    val assumed_invariants_region : Region.t ref
    val assumed_invariants_list_to_region : unit -> Region.t

  end with type Operation.t = C_SD.edge
       and type Operation.exp = Ast.Expression.expression


