(*
 * Copyright  1990-2002 The Regents of the University of California. All rights reserved. 
 *
 * Permission is hereby granted, without written agreement and without 
 * license or royalty fees, to use, copy, modify, and distribute this 
 * software and its documentation for any purpose, provided that the 
 * above copyright notice and the following two paragraphs appear in 
 * all copies of this software. 
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY 
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN 
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. 
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS 
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION 
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *)

(* $Id: message.ml,v 1.3 2003/06/26 20:40:52 jhala Exp $
 *
 * This file is part of the BLAST Project.
 *)


(**
 * This module allows to write messages filtered by their significance.
 *)


type verbosity =
    Quiet
  | Default
  | Verbose
  | Debugging

type significance =
    Error
  | Major
  | Normal
  | Minor
  | Debug

let get_verbosity_level = function
    Quiet -> 0
  | Default -> 1
  | Verbose -> 2
  | Debugging -> 3

let get_significance_level = function
    Error -> -1
  | Major -> 0
  | Normal -> 1
  | Minor -> 2
  | Debug -> 3 (* JHALA HACK HACK HACK! *)

let verbosity_level = ref 0

let set_verbosity v = verbosity_level := (get_verbosity_level v)

let get_verbosity v = 
  let v_l = function
      0 -> Quiet
    | 1 -> Default
    | 2 -> Verbose
    | 3 -> Debugging
    | _ -> Debugging
  in
  v_l (!verbosity_level)

let msg_printer sl printer data =
  let fmt =
    if (sl == Error) then
      Format.err_formatter
    else
      Format.std_formatter
  in
    if (get_significance_level sl) <= !verbosity_level then
      begin
        Format.pp_open_hbox fmt () ;
        printer fmt data ;
        Format.pp_print_newline fmt () ;
        Format.pp_close_box fmt ()
      end
    else
      ()

let msg_string sl str =
  msg_printer sl Format.pp_print_string str
