(*
 * Copyright  1990-2002 The Regents of the University of California. All rights reserved. 
 *
 * Permission is hereby granted, without written agreement and without 
 * license or royalty fees, to use, copy, modify, and distribute this 
 * software and its documentation for any purpose, provided that the 
 * above copyright notice and the following two paragraphs appear in 
 * all copies of this software. 
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY 
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN 
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. 
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS 
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION 
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *)

(* $Id: message.mli,v 1.2 2002/10/18 19:50:25 jhala Exp $
 *
 * This file is part of the BLAST Project.
 *)


 (**
   * This module allows to write messages filtered by their significance.
   *)


type verbosity =
    Quiet      (** only error and major messages shall be displayed *)
  | Default    (** only error, major and normal messages shall be displayed *)
  | Verbose    (** all messages except debug messages shall be displayed *)
  | Debugging  (** all messages shall be displayed *)

type significance =
    Error     (** Error: for error messages *)
  | Major     (** Major: for crucial messages *)
  | Normal    (** Normal: for normal messages *)
  | Minor     (** Minor: for minor messages *)
  | Debug     (** Debug: for debug messages *)

(** sets the verbosity used subsequently by the msg function *)
val set_verbosity : verbosity -> unit

(** get the verbosity level *)
val get_verbosity : unit -> verbosity

(** displays a message according to its significance and to the current
   verbosity level *)
val msg_printer : significance -> (Format.formatter -> 'a -> unit) -> 'a -> unit

(** displays a message string according to its significance and to the current
   verbosity level *)
val msg_string  : significance -> string -> unit
