(*
 * Copyright  1990-2002 The Regents of the University of California. All rights reserved. 
 *
 * Permission is hereby granted, without written agreement and without 
 * license or royalty fees, to use, copy, modify, and distribute this 
 * software and its documentation for any purpose, provided that the 
 * above copyright notice and the following two paragraphs appear in 
 * all copies of this software. 
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY 
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN 
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. 
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS 
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION 
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *)

(* $Id: search.ml,v 1.1 2003/06/05 06:54:08 rupak Exp $
 *
 * This file is part of the BLAST Project.
 *)


  (* this class defines the search strategy *)
  class virtual ['a] iterator =
    object (self)
    method virtual add_element : 'a -> unit
    method virtual add_list : 'a list -> unit
    method virtual next : unit -> 'a
    method virtual has_next : unit -> bool

    method virtual iter : ('a -> unit) -> unit

    method virtual sizeof : unit -> int
    method virtual empty_out : unit -> unit
  end

  class ['a] dfsiterator =
  object (self)
    inherit ['a] iterator

    val mutable q = ([] : 'a list)
    method add_element e = q <- e::q
    method add_list l = List.iter (function e -> self#add_element e) l
    method next () =  match q with
        [] -> raise Not_found
      | h::t -> (q <- t ; h)

    method iter f = List.iter f q
    
    method has_next () = q != []

    method sizeof () = List.length q
	
    method empty_out () = q <- []
  end

  class ['a] bfsiterator =
    object (self)
      
    inherit ['a] iterator

    val mutable q = Queue.create ()
    method add_element e = 
      Queue.add e q
    method add_list l = List.iter (function e -> self#add_element e) l
	
    method iter f = Queue.iter f q

    method next () = Queue.take q
    
    method has_next () = try Queue.peek q; true with Queue.Empty -> false

    method sizeof () = Queue.length q

    method empty_out () = Queue.clear q
  end
