(*
 * Description:
 *
 * Copyright (c) 2000 by
 *  David Blei			blei@cs.berkeley.edu
 *  Chris Harrelson		chrishtr@cs.berkeley.edu
 *  Ranjit Jhala		jhala@cs.berkeley.edu
 *  Rupak Majumdar		rupak@cs.berkeley.edu
 *  George C. Necula	necula@cs.berkeley.edu
 *  Shree P. Rahul		sprahul@cs.berkeley.edu
 * 	Wes Weimer			weimer@cs.berkeley.edu
 *  Dror Weitz			dror@cs.berkeley.edu
 *   
 * All rights reserved.  Permission to use, copy, modify and distribute
 * this software for research purposes only is hereby granted, 
 * provided that the following conditions are met: 
 * 1.  Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 * 3. The name of the authors may not be used to endorse or promote products derived from 
 * this software without specific prior written permission. 
 *
 * DISCLAIMER:
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

(* This defines data types for the output of the parser. *)

(* bound variables *)
type var = string
val  newVar   : string -> var
val  varName  : var -> string

(* predicates *)
type pred =
    True			(* true       *)
  | False			(* false      *)

  | All   of var * pred	        (* forall v.P *)
  | Exist of var * pred	        (* exist v.P  *)
  | And  of pred * pred	        (* P /\ Q     *)
  | Or   of pred * pred	        (* P \/ Q     *)
  | Not of pred			(* ~ P	      *)
  | Imp  of pred * pred	        (* P => Q     *)

  | Comment of string * pred	(* attach a comment to any predicate *)

  | Atom of atom

and atom = 
  | Beq of atom * atom          (* JHALA *)
  | Eq of exp * exp             (* atoms *)
  | Neq of exp * exp
  | Geq of exp * exp
  | Gt of exp * exp
  | Leq of exp * exp
  | Lt of exp * exp

  | BFun  of string * exp list		(*   all other boolean functions   *)
  | NotBFun of string * exp list        (* negated literals *)


and exp =                               (* Expressions *)
    Var of var                          (* Variables *)
  | Int of int                          (* Constants *)
  | Plus of exp * exp
  | Minus of exp * exp
  | Times of exp * exp

                                        (* Some expressions for modular 
                                         * arithmetic *)
  | Div   of int * exp                  (*   division by int  *)
  | Mod   of int * exp                  (*   modulo by int    *)
  | Floor of int * exp                  (*   floor to int     *)
  | Ceil  of int * exp                  (*   ceiling to int   *)
  | Fun   of string * exp list		(*   all other non-bool functions  *)

    
type input =
    Goal   of string * pred
  | Axiom  of string * var list * pred list
  | Import of string
  | Decls  of (string * string list) list

(* Negate an atom *)
val negate : atom -> atom

(* Equivalence checker *)
val samee : exp -> exp -> bool         (* Checks that two expressions are the 
                                        * same *)
val samep : pred -> pred -> bool        (* Checks that two predicates are the 
                                         * same *)

(* push all negations to the Atoms *)
val pushNeg : pred -> pred

                                        (* VampyrePretty printing functions. Have 
                                         * the dummy argument to be usable 
                                         * with VampyrePretty.printf *)
val de  : unit -> exp -> VampyrePretty.doc
val da  : unit -> atom -> VampyrePretty.doc
val dp  : unit -> pred -> VampyrePretty.doc
val di  : unit -> input -> VampyrePretty.doc

val printLFSyntax : bool ref            (* If true then things are printed in 
                                         * the prefix notation of LF *)
