(*
 * Description:
 *
 * Copyright (c) 2000 by
 *  David Blei			blei@cs.berkeley.edu
 *  Chris Harrelson		chrishtr@cs.berkeley.edu
 *  Ranjit Jhala		jhala@cs.berkeley.edu
 *  Rupak Majumdar		rupak@cs.berkeley.edu
 *  George C. Necula	necula@cs.berkeley.edu
 *  Shree P. Rahul		sprahul@cs.berkeley.edu
 * 	Wes Weimer			weimer@cs.berkeley.edu
 *  Dror Weitz			dror@cs.berkeley.edu
 *   
 * All rights reserved.  Permission to use, copy, modify and distribute
 * this software for research purposes only is hereby granted, 
 * provided that the following conditions are met: 
 * 1.  Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 * 3. The name of the authors may not be used to endorse or promote products derived from 
 * this software without specific prior written permission. 
 *
 * DISCLAIMER:
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)


type infot = Equality of int * int | Literal of int
    (* [Jhala] because the damn compiler is whining ... *)

type proofInfo =
    { body : infot ;
      owner : string ;
      token : int ;
      goal : bool;
    }
type eqClassNode =
    { idt : int;
      mutable parent : eqClassNode;
      mutable size : int;
      mutable proof : int*int;
      mutable sP : proofInfo
    }
      
let dummyMesg = {body = Literal 0;owner ="CC";token = 0; goal = false}
      
let newNode i =
  let rec node = {idt = i; parent = node;size = 1;proof = (-1,-1);sP = dummyMesg} in
  node

let swap (x,y) = (y,x)
    
let link t1 t2 =
  if (t1.size > t2.size)
  then (t2.parent <- t1; t1.size <- t1.size + t2.size;t1)
  else (t1.parent <- t2;t2.size <- t1.size + t2.size;t2)

let linkWith t1 t2 who info =
  if (t1.size > t2.size)
  then (t2.parent <- t1; t2.proof <- who; t2.sP <- info; t1.size <- t1.size + t2.size;t1)
  else (t1.parent <- t2; t1.proof <- (swap who); t1.sP <- info; t2.size <- t1.size + t2.size;t2)

      
(* here t2 points to t1 *)
let cut t1 t2 =
  if (t2.parent = t1)
  then
    (
    t2.parent <- t2;
    t1.size <- t1.size - t2.size)
  else (
    print_string "CC: UF: cut invariant broken !! \n";
    (if (t1.parent = t2) then
      (
      t1.parent <- t1;
      t2.size <- t2.size - t1.size)
    else (print_string "CC: UF: cut invariant broken !! \n");
     )
      )
	
      
	
  
let rec rootOf node =
  if (node = node.parent)
  then node
  else (rootOf (node.parent))
     
  (* Path Compression *)
  (*  (
    let p = rootOf node.parent in
    node.parent <- p;
    p
      )
    *)
      
let union n1 n2 =
  let r1 = rootOf n1 in
  let r2 = rootOf n2 in
  if (r1 = r2) then r1
      else (link r1 r2)

let tr_union n1 n2 info =
  let r1 = rootOf n1 in
  let r2 = rootOf n2 in
  if (r1 = r2) then (r1,r1)
  else (
    let foo1 = linkWith r1 r2 (n1.idt,n2.idt) info in
    let foo2 = if (foo1 = r1) then r2 else r1 in
    (foo1,foo2))
      
      
let find node = (rootOf node).idt 
      
let rec print_path node = print_int node.idt;print_string "->";if node.parent = node then () else print_path node.parent
  
let rec onPathtoRoot x y =
  if ((y = x) || (y = x.parent)) then true
  else (
    if x.parent = x then false
    else onPathtoRoot (x.parent) y
	)

